@@ -28,11 +28,25 @@
 #include "foundry-cli-command-private.h"
 #include "foundry-cli-command-tree.h"
 #include "foundry-command-line-private.h"
+#include "foundry-command-line-input-private.h"
 #include "foundry-command-line-local-private.h"
 #include "foundry-command-line-remote-private.h"
 #include "foundry-ipc.h"
+#include "foundry-init-private.h"
+#include "foundry-tty-auth-provider.h"
 #include "foundry-util-private.h"
 
+/**
+ * FoundryCommandLine:
+ *
+ * Abstract base class for command line interface implementations.
+ *
+ * FoundryCommandLine provides the core interface for command line operations including
+ * command parsing, execution, and communication with remote services. Concrete
+ * implementations handle different command line modes and provide unified access
+ * to Foundry functionality from the terminal.
+ */
+
 G_DEFINE_ABSTRACT_TYPE (FoundryCommandLine, foundry_command_line, G_TYPE_OBJECT)
 G_DEFINE_QUARK (foundry_command_line_error, foundry_command_line_error)
 G_DEFINE_ENUM_TYPE (FoundryObjectSerializerFormat, foundry_object_serializer_format,
@@ -111,6 +125,9 @@ foundry_command_line_real_run (FoundryCommandLine *self,
       return dex_future_new_for_int (EXIT_SUCCESS);
     }
 
+  /* Be sure our plugins have loaded */
+  _foundry_init_plugins ();
+
   tree = foundry_cli_command_tree_get_default ();
 
   if (!(command = foundry_cli_command_tree_lookup (tree, &args, &options, &error)))
@@ -244,6 +261,10 @@ foundry_command_line_help (FoundryCommandLine *self)
   foundry_command_line_print (self, "  doc bundle install   %s\n", _("Install a specific documentation bundle"));
   foundry_command_line_print (self, "  doc query            %s\n", _("Search for documentation"));
   foundry_command_line_print (self, "\n");
+  foundry_command_line_print (self, "%s:\n", _("Configuration Commands"));
+  foundry_command_line_print (self, "  config list          %s\n", _("List available configurations"));
+  foundry_command_line_print (self, "  config switch        %s\n", _("Change the active configuration"));
+  foundry_command_line_print (self, "\n");
   foundry_command_line_print (self, "%s:\n", _("SDK Commands"));
   foundry_command_line_print (self, "  sdk list             %s\n", _("List available SDKs"));
   foundry_command_line_print (self, "  sdk install          %s\n", _("Install a specific SDK"));
@@ -255,7 +276,7 @@ foundry_command_line_help (FoundryCommandLine *self)
   foundry_command_line_print (self, "  pipeline rebuild     %s\n", _("Rebuild the current pipeline"));
   foundry_command_line_print (self, "  pipeline clean       %s\n", _("Clean the current pipeline"));
   foundry_command_line_print (self, "  pipeline purge       %s\n", _("Delete contents related to build"));
-  foundry_command_line_print (self, "  pipeline configure   %s\n", _("Delete contents related to build"));
+  foundry_command_line_print (self, "  pipeline configure   %s\n", _("Run pipeline through configure phase"));
   //foundry_command_line_print (self, "  pipeline insert      %s\n", _("Insert a command into the pipeline"));
   foundry_command_line_print (self, "\n");
   foundry_command_line_print (self, "%s:\n", _("Device Commands"));
@@ -416,7 +437,9 @@ typedef struct _Column
   guint       is_boolean : 1;
   guint       is_number : 1;
   guint       is_enum : 1;
+  guint       is_flags : 1;
   guint       is_strv : 1;
+  guint       is_datetime : 1;
 } Column;
 
 static void
@@ -477,9 +500,12 @@ foundry_command_line_print_list (FoundryCommandLine                 *self,
   guint n_columns;
 
   g_assert (FOUNDRY_IS_COMMAND_LINE (self));
-  g_assert (G_IS_LIST_MODEL (model));
+  g_assert (!model || G_IS_LIST_MODEL (model));
   g_assert (entries != NULL);
 
+  if (model == NULL)
+    return;
+
   chunk = g_string_chunk_new (4096);
   strings = g_ptr_array_new ();
 
@@ -507,7 +533,9 @@ foundry_command_line_print_list (FoundryCommandLine                 *self,
           return;
         }
 
+      columns[c].is_datetime = g_type_is_a (columns[c].pspec->value_type, G_TYPE_DATE_TIME);
       columns[c].is_enum = G_TYPE_IS_ENUM (columns[c].pspec->value_type);
+      columns[c].is_flags = G_TYPE_IS_FLAGS (columns[c].pspec->value_type);
       columns[c].is_strv = columns[c].pspec->value_type == G_TYPE_STRV;
       columns[c].is_boolean = columns[c].pspec->value_type == G_TYPE_BOOLEAN;
       columns[c].is_number = is_number_type (columns[c].pspec->value_type);
@@ -523,6 +551,7 @@ foundry_command_line_print_list (FoundryCommandLine                 *self,
         {
           Column *column = &columns[ii];
           g_auto(GValue) value = G_VALUE_INIT;
+          g_autofree char *tmpstr = NULL;
           const char *str;
 
           if (column->is_boolean)
@@ -540,6 +569,23 @@ foundry_command_line_print_list (FoundryCommandLine                 *self,
                     str = g_string_chunk_insert_const (chunk, _("No"));
                 }
             }
+          else if (column->is_datetime)
+            {
+              GDateTime *dt;
+
+              g_value_init (&value, G_TYPE_DATE_TIME);
+              g_object_get_property (object, entries[ii].property, &value);
+
+              if ((dt = g_value_get_boxed (&value)))
+                {
+                  if (format == FOUNDRY_OBJECT_SERIALIZER_FORMAT_JSON)
+                    tmpstr = g_date_time_format_iso8601 (dt);
+                  else
+                    tmpstr = g_date_time_format (dt, "%x %X");
+                }
+
+              str = tmpstr;
+            }
           else if (column->is_enum)
             {
               g_autoptr(GEnumClass) enum_class = g_type_class_ref (column->pspec->value_type);
@@ -553,6 +599,37 @@ foundry_command_line_print_list (FoundryCommandLine                 *self,
               else
                 str = NULL;
             }
+          else if (column->is_flags)
+            {
+              g_autoptr(GFlagsClass) flags_class = g_type_class_ref (column->pspec->value_type);
+              g_autoptr(GString) gstr = g_string_new (NULL);
+              GFlagsValue *v;
+              guint flags;
+
+              g_value_init (&value, column->pspec->value_type);
+              g_object_get_property (object, entries[ii].property, &value);
+
+              flags = g_value_get_flags (&value);
+
+              while ((v = g_flags_get_first_value (flags_class, flags)))
+                {
+                  if (gstr->len > 0)
+                    g_string_append_c (gstr, '|');
+
+                  g_string_append (gstr, v->value_nick);
+
+                  flags &= ~v->value;
+
+                  if (flags == 0)
+                    break;
+                }
+
+              g_value_unset (&value);
+              g_value_init (&value, G_TYPE_STRING);
+              g_value_take_string (&value, g_string_free (g_steal_pointer (&gstr), FALSE));
+
+              str = g_value_get_string (&value);
+            }
           else if (column->is_strv)
             {
               g_auto(GStrv) strv = NULL;
@@ -563,14 +640,24 @@ foundry_command_line_print_list (FoundryCommandLine                 *self,
 
               if ((strv = g_value_dup_boxed (&value)))
                 {
+                  if (format == FOUNDRY_OBJECT_SERIALIZER_FORMAT_JSON)
+                    g_string_append (gstr, "[");
+
                   for (guint z = 0; strv[z]; z++)
                     {
-                      g_autofree char *quoted = g_shell_quote (strv[z]);
+                      g_autofree char *escaped = g_strescape (strv[z], NULL);
 
-                      if (gstr->len > 0)
-                        g_string_append_c (gstr, ' ');
-                      g_string_append (gstr, quoted);
+                      if (format == FOUNDRY_OBJECT_SERIALIZER_FORMAT_JSON)
+                        g_string_append_printf (gstr, "\"%s\"", escaped);
+                      else
+                        g_string_append (gstr, escaped);
+
+                      if (strv[z+1] != NULL)
+                        g_string_append (gstr, ", ");
                     }
+
+                  if (format == FOUNDRY_OBJECT_SERIALIZER_FORMAT_JSON)
+                    g_string_append (gstr, "]");
                 }
 
               g_value_unset (&value);
@@ -589,9 +676,17 @@ foundry_command_line_print_list (FoundryCommandLine                 *self,
 
           if (str != NULL)
             {
-              g_autofree char *escaped = g_strescape (str, NULL);
-              column->longest = MAX (column->longest, strlen (escaped));
-              str = g_string_chunk_insert_const (chunk, escaped);
+              if (column->is_strv)
+                {
+                  column->longest = MAX (column->longest, strlen (str));
+                  str = g_string_chunk_insert_const (chunk, str);
+                }
+              else
+                {
+                  g_autofree char *escaped = g_strescape (str, NULL);
+                  column->longest = MAX (column->longest, strlen (escaped));
+                  str = g_string_chunk_insert_const (chunk, escaped);
+                }
             }
 
           g_ptr_array_add (strings, (char *)str);
@@ -652,12 +747,10 @@ foundry_command_line_print_list (FoundryCommandLine                 *self,
 
               foundry_command_line_print (self, "\"%s\": ", column->pspec->name);
 
-              if (column->is_boolean)
+              if (column->is_boolean || column->is_strv || column->is_number)
                 foundry_command_line_print (self, "%s", str);
               else if (str == NULL)
                 foundry_command_line_print (self, "null");
-              else if (column->is_number)
-                foundry_command_line_print (self, "%s", str);
               else
                 foundry_command_line_print (self, "\"%s\"", str);
             }
@@ -740,3 +833,68 @@ foundry_command_line_set_title (FoundryCommandLine *self,
       /* Do Nothing */
     }
 }
+
+/**
+ * foundry_command_line_dup_auth_provider:
+ * @self: a [class@Foundry.CommandLine]
+ *
+ * Gets an auth provider for this command line client if possible.
+ *
+ * Currrently, this only returns an auth provider if the command line
+ * client provides a TTY.
+ *
+ * Returns: (transfer full) (nullable):
+ */
+FoundryAuthProvider *
+foundry_command_line_dup_auth_provider (FoundryCommandLine *self)
+{
+  g_return_val_if_fail (FOUNDRY_IS_COMMAND_LINE (self), NULL);
+
+  if (foundry_command_line_isatty (self))
+    return foundry_tty_auth_provider_new (foundry_command_line_get_stdin (self));
+
+  return NULL;
+}
+
+/**
+ * foundry_command_line_build_file_for_arg:
+ * @self: a [class@Foundry.CommandLine]
+ *
+ * Returns: (transfer full):
+ */
+GFile *
+foundry_command_line_build_file_for_arg (FoundryCommandLine *self,
+                                         const char         *arg)
+{
+  g_return_val_if_fail (FOUNDRY_IS_COMMAND_LINE (self), NULL);
+  g_return_val_if_fail (arg != NULL, NULL);
+
+  if (g_path_is_absolute (arg))
+    return g_file_new_for_path (arg);
+
+  return g_file_new_build_filename (foundry_command_line_get_directory (self), arg, NULL);
+}
+
+/**
+ * foundry_command_line_request_input:
+ * @self: a [class@Foundry.CommandLine]
+ * @input: a [class@Foundry.Input]
+ *
+ * Queries the user for the information requested in @input.
+ *
+ * Returns: (transfer full): a [class@Dex.Future] that resolves to
+ *   any value or rejects with error.
+ */
+DexFuture *
+foundry_command_line_request_input (FoundryCommandLine *self,
+                                    FoundryInput       *input)
+{
+  int pty_fd;
+
+  dex_return_error_if_fail (FOUNDRY_IS_COMMAND_LINE (self));
+  dex_return_error_if_fail (FOUNDRY_IS_INPUT (input));
+
+  pty_fd = foundry_command_line_get_stdout (self);
+
+  return foundry_command_line_input (pty_fd, input);
+}
