@@ -27,7 +27,7 @@
 #define G_SETTINGS_ENABLE_BACKEND
 #include <gio/gsettingsbackend.h>
 
-#include "foundry-action-muxer-private.h"
+#include "foundry-action-muxer.h"
 #include "foundry-build-manager.h"
 #include "foundry-command-manager.h"
 #include "foundry-config.h"
@@ -35,25 +35,68 @@
 #include "foundry-context-private.h"
 #include "foundry-dbus-service.h"
 #include "foundry-debug.h"
-#include "foundry-debugger-manager.h"
 #include "foundry-dependency-manager.h"
 #include "foundry-device-manager.h"
 #include "foundry-diagnostic-manager.h"
-#include "foundry-documentation-manager.h"
 #include "foundry-file-manager.h"
 #include "foundry-init-private.h"
+#include "foundry-intent-manager.h"
+#include "foundry-license.h"
 #include "foundry-log-manager-private.h"
-#include "foundry-lsp-manager.h"
 #include "foundry-operation-manager.h"
 #include "foundry-run-manager.h"
 #include "foundry-sdk-manager.h"
 #include "foundry-search-manager.h"
+#include "foundry-secret-service.h"
 #include "foundry-service-private.h"
 #include "foundry-settings.h"
-#include "foundry-text-manager.h"
-#include "foundry-vcs-manager.h"
+#include "foundry-test-manager.h"
+#include "foundry-tweak-manager.h"
 #include "foundry-util-private.h"
 
+#ifdef FOUNDRY_FEATURE_DOCS
+# include "foundry-documentation-manager.h"
+#endif
+
+#ifdef FOUNDRY_FEATURE_DEBUGGER
+# include "foundry-debugger-manager.h"
+#endif
+
+#ifdef FOUNDRY_FEATURE_FORGE
+# include "foundry-forge-manager.h"
+#endif
+
+#ifdef FOUNDRY_FEATURE_LLM
+# include "foundry-llm-manager.h"
+#endif
+
+#ifdef FOUNDRY_FEATURE_LSP
+# include "foundry-lsp-manager.h"
+#endif
+
+#ifdef FOUNDRY_FEATURE_TEXT
+# include "foundry-text-manager.h"
+#endif
+
+#ifdef FOUNDRY_FEATURE_TERMINAL
+# include "foundry-terminal-service.h"
+#endif
+
+#ifdef FOUNDRY_FEATURE_VCS
+# include "foundry-vcs-manager.h"
+#endif
+
+/**
+ * FoundryContext:
+ *
+ * The central object representing an open project.
+ *
+ * FoundryContext serves as the main coordinator for all Foundry services and
+ * provides access to managers for files, builds, debugging, and other development
+ * tools. It maintains the application state and facilitates communication between
+ * different components of the development environment.
+ */
+
 struct _FoundryContext
 {
   GObject            parent_instance;
@@ -80,22 +123,46 @@ enum {
   PROP_BUILD_SYSTEM,
   PROP_COMMAND_MANAGER,
   PROP_CONFIG_MANAGER,
+#ifdef FOUNDRY_FEATURE_DEBUGGER
+  PROP_DEBUGGER_MANAGER,
+#endif
   PROP_DEPENDENCY_MANAGER,
   PROP_DEVICE_MANAGER,
   PROP_DIAGNOSTIC_MANAGER,
+#ifdef FOUNDRY_FEATURE_DOCS
   PROP_DOCUMENTATION_MANAGER,
+#endif
   PROP_FILE_MANAGER,
+  PROP_INTENT_MANAGER,
+#ifdef FOUNDRY_FEATURE_FORGE
+  PROP_FORGE_MANAGER,
+#endif
+#ifdef FOUNDRY_FEATURE_LLM
+  PROP_LLM_MANAGER,
+#endif
   PROP_LOG_MANAGER,
+#ifdef FOUNDRY_FEATURE_LSP
   PROP_LSP_MANAGER,
+#endif
   PROP_OPERATION_MANAGER,
   PROP_PROJECT_DIRECTORY,
   PROP_RUN_MANAGER,
   PROP_SDK_MANAGER,
   PROP_SEARCH_MANAGER,
+  PROP_SECRET_SERVICE,
   PROP_STATE_DIRECTORY,
+  PROP_TEST_MANAGER,
+#ifdef FOUNDRY_FEATURE_TERMINAL
+  PROP_TERMINAL_SERVICE,
+#endif
+#ifdef FOUNDRY_FEATURE_TEXT
   PROP_TEXT_MANAGER,
+#endif
   PROP_TITLE,
+  PROP_TWEAK_MANAGER,
+#ifdef FOUNDRY_FEATURE_VCS
   PROP_VCS_MANAGER,
+#endif
   N_PROPS
 };
 
@@ -200,6 +267,12 @@ foundry_context_get_property (GObject    *object,
       g_value_take_object (value, foundry_context_dup_config_manager (self));
       break;
 
+#ifdef FOUNDRY_FEATURE_DEBUGGER
+    case PROP_DEBUGGER_MANAGER:
+      g_value_take_object (value, foundry_context_dup_debugger_manager (self));
+      break;
+#endif
+
     case PROP_DEPENDENCY_MANAGER:
       g_value_take_object (value, foundry_context_dup_dependency_manager (self));
       break;
@@ -212,21 +285,40 @@ foundry_context_get_property (GObject    *object,
       g_value_take_object (value, foundry_context_dup_diagnostic_manager (self));
       break;
 
+#ifdef FOUNDRY_FEATURE_DOCS
     case PROP_DOCUMENTATION_MANAGER:
       g_value_take_object (value, foundry_context_dup_documentation_manager (self));
       break;
+#endif
 
     case PROP_FILE_MANAGER:
       g_value_take_object (value, foundry_context_dup_file_manager (self));
       break;
+    case PROP_INTENT_MANAGER:
+      g_value_take_object (value, foundry_context_dup_intent_manager (self));
+      break;
+
+#ifdef FOUNDRY_FEATURE_FORGE
+    case PROP_FORGE_MANAGER:
+      g_value_take_object (value, foundry_context_dup_forge_manager (self));
+      break;
+#endif
+
+#ifdef FOUNDRY_FEATURE_LLM
+    case PROP_LLM_MANAGER:
+      g_value_take_object (value, foundry_context_dup_llm_manager (self));
+      break;
+#endif
 
     case PROP_LOG_MANAGER:
       g_value_take_object (value, foundry_context_dup_log_manager (self));
       break;
 
+#ifdef FOUNDRY_FEATURE_LSP
     case PROP_LSP_MANAGER:
       g_value_take_object (value, foundry_context_dup_lsp_manager (self));
       break;
+#endif
 
     case PROP_OPERATION_MANAGER:
       g_value_take_object (value, foundry_context_dup_operation_manager (self));
@@ -248,21 +340,43 @@ foundry_context_get_property (GObject    *object,
       g_value_take_object (value, foundry_context_dup_search_manager (self));
       break;
 
+    case PROP_SECRET_SERVICE:
+      g_value_take_object (value, foundry_context_dup_secret_service (self));
+      break;
+
     case PROP_STATE_DIRECTORY:
       g_value_take_object (value, foundry_context_dup_state_directory (self));
       break;
 
+    case PROP_TEST_MANAGER:
+      g_value_take_object (value, foundry_context_dup_test_manager (self));
+      break;
+
+#ifdef FOUNDRY_FEATURE_TERMINAL
+    case PROP_TERMINAL_SERVICE:
+      g_value_take_object (value, foundry_context_dup_terminal_service (self));
+      break;
+#endif
+
+#ifdef FOUNDRY_FEATURE_TEXT
     case PROP_TEXT_MANAGER:
       g_value_take_object (value, foundry_context_dup_text_manager (self));
       break;
+#endif
 
     case PROP_TITLE:
       g_value_take_string (value, foundry_context_dup_title (self));
       break;
 
+    case PROP_TWEAK_MANAGER:
+      g_value_take_object (value, foundry_context_dup_tweak_manager (self));
+      break;
+
+#ifdef FOUNDRY_FEATURE_VCS
     case PROP_VCS_MANAGER:
       g_value_take_object (value, foundry_context_dup_vcs_manager (self));
       break;
+#endif
 
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -322,6 +436,14 @@ foundry_context_class_init (FoundryContextClass *klass)
                          (G_PARAM_READABLE |
                           G_PARAM_STATIC_STRINGS));
 
+#ifdef FOUNDRY_FEATURE_DEBUGGER
+  properties[PROP_DEBUGGER_MANAGER] =
+    g_param_spec_object ("debugger-manager", NULL, NULL,
+                         FOUNDRY_TYPE_DEBUGGER_MANAGER,
+                         (G_PARAM_READABLE |
+                          G_PARAM_STATIC_STRINGS));
+#endif
+
   properties[PROP_DEPENDENCY_MANAGER] =
     g_param_spec_object ("dependency-manager", NULL, NULL,
                          FOUNDRY_TYPE_DEPENDENCY_MANAGER,
@@ -340,11 +462,13 @@ foundry_context_class_init (FoundryContextClass *klass)
                          (G_PARAM_READABLE |
                           G_PARAM_STATIC_STRINGS));
 
+#ifdef FOUNDRY_FEATURE_DOCS
   properties[PROP_DOCUMENTATION_MANAGER] =
     g_param_spec_object ("documentation-manager", NULL, NULL,
                          FOUNDRY_TYPE_DOCUMENTATION_MANAGER,
                          (G_PARAM_READABLE |
                           G_PARAM_STATIC_STRINGS));
+#endif
 
   properties[PROP_FILE_MANAGER] =
     g_param_spec_object ("file-manager", NULL, NULL,
@@ -352,17 +476,46 @@ foundry_context_class_init (FoundryContextClass *klass)
                          (G_PARAM_READABLE |
                           G_PARAM_STATIC_STRINGS));
 
+  properties[PROP_INTENT_MANAGER] =
+    g_param_spec_object ("intent-manager", NULL, NULL,
+                         FOUNDRY_TYPE_INTENT_MANAGER,
+                         (G_PARAM_READABLE |
+                          G_PARAM_STATIC_STRINGS));
+
+#ifdef FOUNDRY_FEATURE_FORGE
+  /**
+   * FoundryContext:forge-manager:
+   *
+   * Since: 1.1
+   */
+  properties[PROP_FORGE_MANAGER] =
+    g_param_spec_object ("forge-manager", NULL, NULL,
+                         FOUNDRY_TYPE_FORGE_MANAGER,
+                         (G_PARAM_READABLE |
+                          G_PARAM_STATIC_STRINGS));
+#endif
+
+#ifdef FOUNDRY_FEATURE_LLM
+  properties[PROP_LLM_MANAGER] =
+    g_param_spec_object ("llm-manager", NULL, NULL,
+                         FOUNDRY_TYPE_LLM_MANAGER,
+                         (G_PARAM_READABLE |
+                          G_PARAM_STATIC_STRINGS));
+#endif
+
   properties[PROP_LOG_MANAGER] =
     g_param_spec_object ("log-manager", NULL, NULL,
                          FOUNDRY_TYPE_LOG_MANAGER,
                          (G_PARAM_READABLE |
                           G_PARAM_STATIC_STRINGS));
 
+#ifdef FOUNDRY_FEATURE_LSP
   properties[PROP_LSP_MANAGER] =
     g_param_spec_object ("lsp-manager", NULL, NULL,
                          FOUNDRY_TYPE_LSP_MANAGER,
                          (G_PARAM_READABLE |
                           G_PARAM_STATIC_STRINGS));
+#endif
 
   properties[PROP_OPERATION_MANAGER] =
     g_param_spec_object ("operation-manager", NULL, NULL,
@@ -400,6 +553,12 @@ foundry_context_class_init (FoundryContextClass *klass)
                          (G_PARAM_READABLE |
                           G_PARAM_STATIC_STRINGS));
 
+  properties[PROP_SECRET_SERVICE] =
+    g_param_spec_object ("secret-service", NULL, NULL,
+                         FOUNDRY_TYPE_SECRET_SERVICE,
+                         (G_PARAM_READABLE |
+                          G_PARAM_STATIC_STRINGS));
+
   /**
    * FoundryContext:state-directory:
    *
@@ -411,11 +570,33 @@ foundry_context_class_init (FoundryContextClass *klass)
                          G_TYPE_FILE,
                          (G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
+  properties[PROP_TEST_MANAGER] =
+    g_param_spec_object ("test-manager", NULL, NULL,
+                         FOUNDRY_TYPE_TEST_MANAGER,
+                         (G_PARAM_READABLE |
+                          G_PARAM_STATIC_STRINGS));
+
+#ifdef FOUNDRY_FEATURE_TERMINAL
+  properties[PROP_TERMINAL_SERVICE] =
+    g_param_spec_object ("terminal-service", NULL, NULL,
+                         FOUNDRY_TYPE_TERMINAL_SERVICE,
+                         (G_PARAM_READABLE |
+                          G_PARAM_STATIC_STRINGS));
+#endif
+
+#ifdef FOUNDRY_FEATURE_TEXT
   properties[PROP_TEXT_MANAGER] =
     g_param_spec_object ("text-manager", NULL, NULL,
                          FOUNDRY_TYPE_TEXT_MANAGER,
                          (G_PARAM_READABLE |
                           G_PARAM_STATIC_STRINGS));
+#endif
+
+  properties[PROP_TWEAK_MANAGER] =
+    g_param_spec_object ("tweak-manager", NULL, NULL,
+                         FOUNDRY_TYPE_TWEAK_MANAGER,
+                         (G_PARAM_READABLE |
+                          G_PARAM_STATIC_STRINGS));
 
   properties[PROP_TITLE] =
     g_param_spec_string ("title", NULL, NULL,
@@ -424,11 +605,13 @@ foundry_context_class_init (FoundryContextClass *klass)
                           G_PARAM_EXPLICIT_NOTIFY |
                           G_PARAM_STATIC_STRINGS));
 
+#ifdef FOUNDRY_FEATURE_VCS
   properties[PROP_VCS_MANAGER] =
     g_param_spec_object ("vcs-manager", NULL, NULL,
                          FOUNDRY_TYPE_VCS_MANAGER,
                          (G_PARAM_READABLE |
                           G_PARAM_STATIC_STRINGS));
+#endif
 
   g_object_class_install_properties (object_class, N_PROPS, properties);
 }
@@ -465,14 +648,22 @@ foundry_context_init (FoundryContext *self)
                    g_object_new (FOUNDRY_TYPE_CONFIG_MANAGER,
                                  "context", self,
                                  NULL));
+#ifdef FOUNDRY_FEATURE_DEBUGGER
+  g_ptr_array_add (self->services,
+                   g_object_new (FOUNDRY_TYPE_DEBUGGER_MANAGER,
+                                 "context", self,
+                                 NULL));
+#endif
   g_ptr_array_add (self->services,
                    g_object_new (FOUNDRY_TYPE_DEPENDENCY_MANAGER,
                                  "context", self,
                                  NULL));
+#ifdef FOUNDRY_FEATURE_DOCS
   g_ptr_array_add (self->services,
                    g_object_new (FOUNDRY_TYPE_DOCUMENTATION_MANAGER,
                                  "context", self,
                                  NULL));
+#endif
   g_ptr_array_add (self->services,
                    g_object_new (FOUNDRY_TYPE_DEVICE_MANAGER,
                                  "context", self,
@@ -485,10 +676,28 @@ foundry_context_init (FoundryContext *self)
                    g_object_new (FOUNDRY_TYPE_FILE_MANAGER,
                                  "context", self,
                                  NULL));
+  g_ptr_array_add (self->services,
+                   g_object_new (FOUNDRY_TYPE_INTENT_MANAGER,
+                                 "context", self,
+                                 NULL));
+#ifdef FOUNDRY_FEATURE_FORGE
+  g_ptr_array_add (self->services,
+                   g_object_new (FOUNDRY_TYPE_FORGE_MANAGER,
+                                 "context", self,
+                                 NULL));
+#endif
+#ifdef FOUNDRY_FEATURE_LLM
+  g_ptr_array_add (self->services,
+                   g_object_new (FOUNDRY_TYPE_LLM_MANAGER,
+                                 "context", self,
+                                 NULL));
+#endif
+#ifdef FOUNDRY_FEATURE_LSP
   g_ptr_array_add (self->services,
                    g_object_new (FOUNDRY_TYPE_LSP_MANAGER,
                                  "context", self,
                                  NULL));
+#endif
   g_ptr_array_add (self->services,
                    g_object_new (FOUNDRY_TYPE_OPERATION_MANAGER,
                                  "context", self,
@@ -497,6 +706,10 @@ foundry_context_init (FoundryContext *self)
                    g_object_new (FOUNDRY_TYPE_RUN_MANAGER,
                                  "context", self,
                                  NULL));
+  g_ptr_array_add (self->services,
+                   g_object_new (FOUNDRY_TYPE_SECRET_SERVICE,
+                                 "context", self,
+                                 NULL));
   g_ptr_array_add (self->services,
                    g_object_new (FOUNDRY_TYPE_SDK_MANAGER,
                                  "context", self,
@@ -505,14 +718,32 @@ foundry_context_init (FoundryContext *self)
                    g_object_new (FOUNDRY_TYPE_SEARCH_MANAGER,
                                  "context", self,
                                  NULL));
+  g_ptr_array_add (self->services,
+                   g_object_new (FOUNDRY_TYPE_TEST_MANAGER,
+                                 "context", self,
+                                 NULL));
+#ifdef FOUNDRY_FEATURE_TERMINAL
+  g_ptr_array_add (self->services,
+                   g_object_new (FOUNDRY_TYPE_TERMINAL_SERVICE,
+                                 "context", self,
+                                 NULL));
+#endif
+#ifdef FOUNDRY_FEATURE_TEXT
   g_ptr_array_add (self->services,
                    g_object_new (FOUNDRY_TYPE_TEXT_MANAGER,
                                  "context", self,
                                  NULL));
+#endif
+  g_ptr_array_add (self->services,
+                   g_object_new (FOUNDRY_TYPE_TWEAK_MANAGER,
+                                 "context", self,
+                                 NULL));
+#ifdef FOUNDRY_FEATURE_VCS
   g_ptr_array_add (self->services,
                    g_object_new (FOUNDRY_TYPE_VCS_MANAGER,
                                  "context", self,
                                  NULL));
+#endif
 
   G_LOCK (all_contexts);
   g_queue_push_head_link (&all_contexts, &self->link);
@@ -689,7 +920,8 @@ foundry_context_load_fiber (FoundryContext  *self,
                                            g_object_unref));
     }
 
-  dex_await (dex_future_allv ((DexFuture **)futures->pdata, futures->len), NULL);
+  if (futures->len > 0)
+    dex_await (dex_future_allv ((DexFuture **)futures->pdata, futures->len), NULL);
 
   return TRUE;
 }
@@ -703,6 +935,7 @@ foundry_context_new_fiber (gpointer data)
   g_autoptr(GFile) project_dir = NULL;
   g_autoptr(GFile) user_dir = NULL;
   g_autoptr(GFile) tmp_dir = NULL;
+  g_autofree char *title = NULL;
 
   g_assert (state != NULL);
   g_assert (G_IS_FILE (state->foundry_dir));
@@ -716,20 +949,23 @@ foundry_context_new_fiber (gpointer data)
   if ((state->flags & (FOUNDRY_CONTEXT_FLAGS_CREATE|FOUNDRY_CONTEXT_FLAGS_SHARED)) != 0)
     {
       g_autoptr(GBytes) bytes = NULL;
+      gboolean setup_ignore = TRUE;
 
-      if (!g_file_make_directory_with_parents (state->foundry_dir, NULL, &error))
+      if (!dex_await (dex_file_make_directory_with_parents (state->foundry_dir), &error))
         {
-          if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_EXISTS) &&
-              (state->flags & FOUNDRY_CONTEXT_FLAGS_SHARED) != 0)
-            goto create_context;
+          if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_EXISTS) ||
+              (state->flags & FOUNDRY_CONTEXT_FLAGS_SHARED) == 0)
+            return dex_future_new_for_error (g_steal_pointer (&error));
 
-          return dex_future_new_for_error (g_steal_pointer (&error));
+          setup_ignore = FALSE;
         }
 
       /* Setup default .gitignore for the .foundry dir */
-      if ((bytes = g_resources_lookup_data ("/app/devsuite/foundry/.foundry/.gitignore", 0, NULL)))
+      if (setup_ignore &&
+          (bytes = g_resources_lookup_data ("/app/devsuite/foundry/.foundry/.gitignore", 0, NULL)))
         {
           g_autoptr(GFile) gitignore = g_file_get_child (state->foundry_dir, ".gitignore");
+
           dex_await (dex_file_replace_contents_bytes (gitignore,
                                                       bytes,
                                                       NULL,
@@ -739,7 +975,6 @@ foundry_context_new_fiber (gpointer data)
         }
     }
 
-create_context:
   self = g_object_new (FOUNDRY_TYPE_CONTEXT, NULL);
   self->is_shared = !!(state->flags & FOUNDRY_CONTEXT_FLAGS_SHARED);
   self->state_directory = g_file_dup (state->foundry_dir);
@@ -748,6 +983,10 @@ create_context:
   if (!foundry_context_load_fiber (self, &error))
     return dex_future_new_for_error (g_steal_pointer (&error));
 
+  title = foundry_context_dup_title (self);
+  /* translators: %s is replaced with project title */
+  FOUNDRY_INFO (self, _("Project “%s” loaded"), title);
+
   return dex_future_new_take_object (g_steal_pointer (&self));
 }
 
@@ -1202,6 +1441,7 @@ foundry_context_dup_diagnostic_manager (FoundryContext *self)
   return foundry_context_dup_service_typed (self, FOUNDRY_TYPE_DIAGNOSTIC_MANAGER);
 }
 
+#ifdef FOUNDRY_FEATURE_DOCS
 /**
  * foundry_context_dup_documentation_manager:
  * @self: a #FoundryContext
@@ -1217,6 +1457,7 @@ foundry_context_dup_documentation_manager (FoundryContext *self)
 
   return foundry_context_dup_service_typed (self, FOUNDRY_TYPE_DOCUMENTATION_MANAGER);
 }
+#endif
 
 /**
  * foundry_context_dup_file_manager:
@@ -1234,6 +1475,41 @@ foundry_context_dup_file_manager (FoundryContext *self)
   return foundry_context_dup_service_typed (self, FOUNDRY_TYPE_FILE_MANAGER);
 }
 
+/**
+ * foundry_context_dup_intent_manager:
+ * @self: a #FoundryContext
+ *
+ * Gets the #FoundryIntentManager instance.
+ *
+ * Returns: (transfer full): a #FoundryIntentManager
+ */
+FoundryIntentManager *
+foundry_context_dup_intent_manager (FoundryContext *self)
+{
+  g_return_val_if_fail (FOUNDRY_IS_CONTEXT (self), NULL);
+
+  return foundry_context_dup_service_typed (self, FOUNDRY_TYPE_INTENT_MANAGER);
+}
+
+#ifdef FOUNDRY_FEATURE_FORGE
+/**
+ * foundry_context_dup_forge_manager:
+ * @self: a #FoundryContext
+ *
+ * Gets the #FoundryForgeManager instance.
+ *
+ * Returns: (transfer full): a #FoundryForgeManager
+ */
+FoundryForgeManager *
+foundry_context_dup_forge_manager (FoundryContext *self)
+{
+  g_return_val_if_fail (FOUNDRY_IS_CONTEXT (self), NULL);
+
+  return foundry_context_dup_service_typed (self, FOUNDRY_TYPE_FORGE_MANAGER);
+}
+#endif
+
+#ifdef FOUNDRY_FEATURE_DEBUGGER
 /**
  * foundry_context_dup_debugger_manager:
  * @self: a #FoundryContext
@@ -1249,6 +1525,25 @@ foundry_context_dup_debugger_manager (FoundryContext *self)
 
   return foundry_context_dup_service_typed (self, FOUNDRY_TYPE_DEBUGGER_MANAGER);
 }
+#endif
+
+#ifdef FOUNDRY_FEATURE_LLM
+/**
+ * foundry_context_dup_llm_manager:
+ * @self: a #FoundryContext
+ *
+ * Gets the #FoundryLlmManager instance.
+ *
+ * Returns: (transfer full): a #FoundryLlmManager
+ */
+FoundryLlmManager *
+foundry_context_dup_llm_manager (FoundryContext *self)
+{
+  g_return_val_if_fail (FOUNDRY_IS_CONTEXT (self), NULL);
+
+  return foundry_context_dup_service_typed (self, FOUNDRY_TYPE_LLM_MANAGER);
+}
+#endif
 
 /**
  * foundry_context_dup_log_manager:
@@ -1266,6 +1561,7 @@ foundry_context_dup_log_manager (FoundryContext *self)
   return foundry_context_dup_service_typed (self, FOUNDRY_TYPE_LOG_MANAGER);
 }
 
+#ifdef FOUNDRY_FEATURE_LSP
 /**
  * foundry_context_dup_lsp_manager:
  * @self: a #FoundryContext
@@ -1281,6 +1577,7 @@ foundry_context_dup_lsp_manager (FoundryContext *self)
 
   return foundry_context_dup_service_typed (self, FOUNDRY_TYPE_LSP_MANAGER);
 }
+#endif
 
 /**
  * foundry_context_dup_operation_manager:
@@ -1330,6 +1627,57 @@ foundry_context_dup_sdk_manager (FoundryContext *self)
   return foundry_context_dup_service_typed (self, FOUNDRY_TYPE_SDK_MANAGER);
 }
 
+/**
+ * foundry_context_dup_secret_service:
+ * @self: a #FoundryContext
+ *
+ * Gets the #FoundrySecretService instance.
+ *
+ * Returns: (transfer full): a #FoundrySecretService
+ */
+FoundrySecretService *
+foundry_context_dup_secret_service (FoundryContext *self)
+{
+  g_return_val_if_fail (FOUNDRY_IS_CONTEXT (self), NULL);
+
+  return foundry_context_dup_service_typed (self, FOUNDRY_TYPE_SECRET_SERVICE);
+}
+
+/**
+ * foundry_context_dup_test_manager:
+ * @self: a #FoundryContext
+ *
+ * Gets the #FoundryTestManager instance.
+ *
+ * Returns: (transfer full): a #FoundryTestManager
+ */
+FoundryTestManager *
+foundry_context_dup_test_manager (FoundryContext *self)
+{
+  g_return_val_if_fail (FOUNDRY_IS_CONTEXT (self), NULL);
+
+  return foundry_context_dup_service_typed (self, FOUNDRY_TYPE_TEST_MANAGER);
+}
+
+#ifdef FOUNDRY_FEATURE_TERMINAL
+/**
+ * foundry_context_dup_terminal_service:
+ * @self: a #FoundryContext
+ *
+ * Gets the #FoundryTerminalService instance.
+ *
+ * Returns: (transfer full): a #FoundryTerminalService
+ */
+FoundryTerminalService *
+foundry_context_dup_terminal_service (FoundryContext *self)
+{
+  g_return_val_if_fail (FOUNDRY_IS_CONTEXT (self), NULL);
+
+  return foundry_context_dup_service_typed (self, FOUNDRY_TYPE_TERMINAL_SERVICE);
+}
+#endif
+
+#ifdef FOUNDRY_FEATURE_TEXT
 /**
  * foundry_context_dup_text_manager:
  * @self: a #FoundryContext
@@ -1345,7 +1693,25 @@ foundry_context_dup_text_manager (FoundryContext *self)
 
   return foundry_context_dup_service_typed (self, FOUNDRY_TYPE_TEXT_MANAGER);
 }
+#endif
 
+/**
+ * foundry_context_dup_tweak_manager:
+ * @self: a #FoundryContext
+ *
+ * Gets the #FoundryTweakManager instance.
+ *
+ * Returns: (transfer full): a #FoundryTweakManager
+ */
+FoundryTweakManager *
+foundry_context_dup_tweak_manager (FoundryContext *self)
+{
+  g_return_val_if_fail (FOUNDRY_IS_CONTEXT (self), NULL);
+
+  return foundry_context_dup_service_typed (self, FOUNDRY_TYPE_TWEAK_MANAGER);
+}
+
+#ifdef FOUNDRY_FEATURE_VCS
 /**
  * foundry_context_dup_vcs_manager:
  * @self: a #FoundryContext
@@ -1361,6 +1727,7 @@ foundry_context_dup_vcs_manager (FoundryContext *self)
 
   return foundry_context_dup_service_typed (self, FOUNDRY_TYPE_VCS_MANAGER);
 }
+#endif
 
 /**
  * foundry_context_dup_search_manager:
@@ -1596,6 +1963,60 @@ foundry_context_cache_filename (FoundryContext *self,
   return g_steal_pointer (&path);
 }
 
+/**
+ * foundry_context_cache_file:
+ * @self: a [class@Foundry.Context]
+ *
+ * Returns: (transfer full): a [iface@Gio.File] within the cache root
+ */
+GFile *
+foundry_context_cache_file (FoundryContext *self,
+                            ...)
+{
+  g_autoptr(GFile) cache_root = NULL;
+  g_autofree char *path = NULL;
+  va_list args;
+
+  g_return_val_if_fail (FOUNDRY_IS_CONTEXT (self), NULL);
+
+  cache_root = foundry_context_dup_cache_root (self);
+
+  va_start (args, self);
+  path = g_build_filename_valist (g_file_peek_path (cache_root), &args);
+  va_end (args);
+
+  return g_file_new_for_path (path);
+}
+
+/**
+ * foundry_context_tmp_filename:
+ * @self: a [class@Foundry.Context]
+ *
+ * Returns a path that will be in the "tmp/" directory of the .foundry dir.
+ *
+ * Returns: (transfer full): a new path to the file
+ *
+ * Since: 1.1
+ */
+char *
+foundry_context_tmp_filename (FoundryContext *self,
+                              ...)
+{
+  g_autoptr(GFile) tmp_root = NULL;
+  g_autofree char *path = NULL;
+  va_list args;
+
+  g_return_val_if_fail (FOUNDRY_IS_CONTEXT (self), NULL);
+
+  tmp_root = g_file_get_child (self->state_directory, "tmp");
+
+  va_start (args, self);
+  path = g_build_filename_valist (g_file_peek_path (tmp_root), &args);
+  va_end (args);
+
+  return g_steal_pointer (&path);
+}
+
 /**
  * foundry_context_dup_build_system:
  * @self: a [class@Foundry.Context]
@@ -1607,6 +2028,10 @@ foundry_context_cache_filename (FoundryContext *self,
  *
  * Otherwise, %NULL is returned.
  *
+ * Build addins can also affect which build system is used and you may
+ * check what the build pipeline thinks should be the build system (which
+ * checks this value) by using [prop@Foundry.BuildPipeline.build-system].
+ *
  * Returns: (transfer full) (nullable): a build-system name or %NULL
  */
 char *
@@ -1621,15 +2046,18 @@ foundry_context_dup_build_system (FoundryContext *self)
 
   settings = foundry_context_load_settings (self, "app.devsuite.foundry.project", NULL);
   build_system = foundry_settings_get_string (settings, "build-system");
+  config_manager = foundry_context_dup_config_manager (self);
 
   if (!foundry_str_empty0 (build_system))
     return g_steal_pointer (&build_system);
 
-  config_manager = foundry_context_dup_config_manager (self);
-  config = foundry_config_manager_dup_config (config_manager);
+  if ((config = foundry_config_manager_dup_config (config_manager)))
+    {
+      g_autofree char *config_build_system = foundry_config_dup_build_system (config);
 
-  if (config != NULL)
-    return foundry_config_dup_build_system (config);
+      if (!foundry_str_empty0 (config_build_system))
+        return g_steal_pointer (&config_build_system);
+    }
 
   return NULL;
 }
@@ -1805,3 +2233,78 @@ foundry_context_dup_action_group (FoundryContext *self)
 
   return G_ACTION_GROUP (g_steal_pointer (&muxer));
 }
+
+/**
+ * foundry_context_list_services:
+ * @self: a [class@Foundry.Context]
+ *
+ * Returns: (transfer full): a [iface@Gio.ListModel] of [class@Foundry.Service]
+ */
+GListModel *
+foundry_context_list_services (FoundryContext *self)
+{
+  GListStore *store;
+
+  g_return_val_if_fail (FOUNDRY_IS_CONTEXT (self), NULL);
+
+  store = g_list_store_new (FOUNDRY_TYPE_SERVICE);
+  g_list_store_splice (store, 0, 0, self->services->pdata, self->services->len);
+  return G_LIST_MODEL (store);
+}
+
+/**
+ * foundry_context_dup_default_license:
+ * @self: a [class@Foundry.Context]
+ *
+ * Returns: (transfer full) (nullable):
+ */
+FoundryLicense *
+foundry_context_dup_default_license (FoundryContext *self)
+{
+  g_autoptr(FoundrySettings) settings = NULL;
+  g_autofree char *default_license = NULL;
+
+  g_return_val_if_fail (FOUNDRY_IS_CONTEXT (self), NULL);
+
+  settings = foundry_context_load_project_settings (self);
+  default_license = foundry_settings_get_string (settings, "default-license");
+
+  if (default_license == NULL || default_license[0] == 0)
+    return NULL;
+
+  return foundry_license_find (default_license);
+}
+
+/**
+ * _foundry_context_find:
+ *
+ * Find an already loaded context for state_directory.
+ *
+ * Returns: (transfer full) (nullable):
+ */
+FoundryContext *
+_foundry_context_find (const char *state_directory)
+{
+  FoundryContext *ret = NULL;
+  g_autoptr(GFile) file = g_file_new_for_path (state_directory);
+
+  G_LOCK (all_contexts);
+
+  for (const GList *iter = all_contexts.head; iter; iter = iter->next)
+    {
+      FoundryContext *context = iter->data;
+
+      if (foundry_context_in_shutdown (context))
+        continue;
+
+      if (g_file_equal (context->state_directory, file))
+        {
+          g_set_object (&ret, context);
+          break;
+        }
+    }
+
+  G_UNLOCK (all_contexts);
+
+  return g_steal_pointer (&ret);
+}
